import os
import logging
from flask import Flask, render_template, abort, jsonify

from p2.tools.json import serializable
from p2.store.client import Client
from p2.logging import HttpLogger
from p2.ui.campaigns.metrics import Metrics

log = HttpLogger(
    level = logging.DEBUG,
    silent = False,
    name='p2.ui.campaigns'
)
logger = log.getLogger()

app = Flask(__name__)
app.config.from_object('p2.ui.campaigns.settings')

p2_store_url = os.getenv('P2_STORE_URL')
if p2_store_url is None:
    raise Exception("No P2_STORE_URL envar found")

client = Client(p2_store_url)

# templates
display_list_t = "campaigns_list.html"

app_route = ''
if __name__ == '__main__':
    app_route = '/campaigns.app'

@app.route(app_route+'/version')
def version():
    return f"p2.ui.campaigns app version: {app.config['VERSION']}"

@app.route(app_route+'/menu')
def menu():
    return jsonify(app.config['MENU'])


@app.route(app_route+'/display_list')
def display_list():
    campaigns = client.select(
        table='test_campaigns',
        sort='id desc',
        fetch=[
            'id',
            'title',
            'start_date',
            'end_date',
            'one_off',
            'status'
        ])
    if campaigns is None:
        raise Exception("Error: no campaigns found")

    template_name = display_list_t
    return render_template(template_name,
                           campaigns=campaigns)

@app.route(app_route+'/api/display_list')
def api_display_list():
    campaigns = client.select(
        table='test_campaigns',
        sort='id desc',
        fetch=[
            'id',
            'title',
            'start_date',
            'end_date',
            'one_off',
            'status'
        ])
    if campaigns is None:
        raise Exception("Error: no campaigns found")
    return jsonify(campaigns)

@app.route('/api/get_campaign_config/<id>')
def get_component_config(id: int):
    logger.debug(f"Getting config sections for campaign: {id}")
    rec = client.select(
        table='test_campaigns',
        fetch=[
            'shifts',
            'resources',
            'experiments',
            'template_params'
        ],
        filter={
            'id': id
        })

    if rec is None or len(rec) != 1:
        abort(404)
    else:
        return jsonify(rec[0])

@app.route('/api/get_metrics/<id>')
def get_metrics(id: int):
    logger.debug(f"Getting metrics for campaign: {id}")

    try:
        # we first get the total number of targets for this campaign...
        # we need to get to the campaign config first and then apply the 
        # targets filter...
        resources_filter = client.select(
            table='test_campaigns',
            fetch=['resources'],
            filter={
                'id': id
            })[0]['resources']

        total_number_of_targets = (client.select(
            table='resources',
            fetch=['count(*)'],
            filter=resources_filter
        ))[0]['count']

        metrics = client.execute_command("""
            SELECT COUNT(*) AS total,
                   COUNT(DISTINCT resource_id) AS targets,
                   COUNT(CASE WHEN status = 'completed' THEN 1 END) AS completed,
                   COUNT(CASE WHEN status = 'failed' THEN 1 END) AS aborted,
                   COUNT(CASE WHEN status = 'deleted' THEN 1 END) AS deleted,
                   COUNT(CASE WHEN status = 'scheduled' THEN 1 END) AS scheduled,
                   COUNT(CASE WHEN test_result=true THEN 1 END) AS passed,
                   COUNT(CASE WHEN test_result=false THEN 1 END) AS failed
            FROM session_report_view
            WHERE campaign_id = %s""", [str(id)])[0]
        metrics_obj = Metrics({
            'total_number_of_tests': metrics['total'],
            'total_number_of_targets': total_number_of_targets,
            'target_coverage': int(metrics['targets'] / total_number_of_targets * 100),
            'perc_completed' : int(metrics['completed'] / metrics['total'] * 100),
            'perc_aborted'   : int(metrics['aborted'] / metrics['total'] * 100),
            'perc_deleted'   : int(metrics['deleted'] / metrics['total'] * 100),
            'perc_scheduled' : int(metrics['scheduled'] / metrics['total'] * 100),
            'perc_passed'    : int(metrics['passed'] / metrics['completed'] * 100),
            'perc_failed'    : int(metrics['failed'] / metrics['completed'] * 100)
        })
        return jsonify(serializable(metrics_obj))
    except Exception as e:
        logger.error(f"Error getting metrics: {e}")
        return jsonify(serializable(Metrics()))



if __name__ == '__main__':
    app.run()
